using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading;
using System.Threading.Tasks;
using ClientServer.Controllers.Core.Exercises;
using ClientServer.Db;
using ClientServer.Helpers;
using ClientServer.Models;
using ClientServer.Models.Exercises.Release;
using ClientServer.Models.Exercises.Solution;
using ClientServer.Models.Exercises.Tests;
using ClientServer.Models.Interfaces;
using ClientServer.Models.Users;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;

namespace ClientServer.Controllers.Core.Testing.ThirdParty
{
    [Route(Constants.ApiPrefix + "thirdpartysqlchecker")]
    public class SqlCheckerHelper : ControllerWithDb
    {
        //TODO fragen submissionId immer > 0 ?
        //TODO soll das ergebnis gespichert werden???
        //TODO handle exceptions
        //TODO vor alle enum namen RESULT_???

        public SqlCheckerHelper(YapexDbContext context) : base(context)
        {
        }


        //TODO copy pasted from TestingController.RequestRunTest
        [HttpPost("run")]
        public async Task RequestCheckerSubmitTest([FromBody] RunTestCommandForBackend command)
        {
            if (!await base.IsLoggedIn()) return;

            int userId = GetUserId();


            if (string.IsNullOrWhiteSpace(command.GeneratedCode))
            {
                await Response.WriteAsync(Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "exercise not found")));
                return;
            }

            var systemSettings = await _context.SystemSettings.FirstOrDefaultAsync();

            if (systemSettings == null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "system settings not found")));
                return;
            }

            var participation = await _context.ExerciseReleaseWithUserAsParticipations
                .Include(p => p.ExerciseRelease)
                .ThenInclude(p => p.Exercise.CodeTemplates)
                .ThenInclude(p => p.TemplateFiles)
                .FirstOrDefaultAsync(p => p.UserId == userId
                                          //&& p.ExerciseRelease.IsReleased // is ReleaseHelper.IsTimeUp checks for is released so wo do not need to double check this
                                          && p.ExerciseRelease.GeneratedCode == command.GeneratedCode);


            if (participation?.ExerciseRelease == null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "no participation or release found")));
                return;
            }

            if (participation.ExerciseRelease.Exercise == null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "exercise not found")));
                return;
            }

            if (participation.ExerciseRelease.Exercise.IsPermanentlyLocked)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.InvalidRequest, "exercise is permanently locked")));
                return;
            }

            if (ReleaseHelper.IsTimeUp(participation, participation.ExerciseRelease, this)) return;


            if (!command.IsCustomTest && !command.IsNormalTest &&
                !command.IsSubmitTest)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.InvalidRequest, "test type not set")));
                return;
            }

            //then we can get the plang directly from the user solution...
            PLang pLangToUse = await _context.PLangs
                .FirstOrDefaultAsync(p => p.Id == command.PlangId);

            if (pLangToUse == null)
            {
                await Response.WriteAsync(Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "p lang not found")));
                return;
            }

            //which test kind??
            if (command.IsNormalTest)
            {
                await _RunNormalOrSubmitTest(command, userId, pLangToUse, participation.ExerciseRelease, false,
                    systemSettings);
            }
            else if (command.IsCustomTest)
            {
//                await _RunCustomTest(command, userId, pLangToUse, participation.ExerciseRelease, systemSettings);
                await Response.WriteAsync(Jc.Serialize(new BasicResponse(ResponseCode.ServerError, "not implemented")));
            }
            else if (command.IsSubmitTest)
            {
                Console.WriteLine("[WARNING] running submit tests is not supported here (yet)");

                //not used from frontend... user cannot run submit tests... maybe later for tutors?
//                await _RunNormalOrSubmitTest(command, userId, pLangToUse, participation.ExerciseRelease, true, systemSettings);
            }
        }


        //TODO copy pasted from TestingController._RunNormalOrSubmitTest
        private async Task _RunNormalOrSubmitTest(RunTestCommandForBackend runNormalTestCommand, int userId,
            PLang plang, ExerciseRelease exerciseRelease, bool isSubmitTest, SystemSetting systemSettings)
        {
            if (isSubmitTest)
            {
                //the user group that manages the exercise
                var targetUserGroup =
                    await _context.Exercises.Where(p => p.Id == exerciseRelease.ExerciseId)
                        .Select(p => p.UserGroupId)
                        .FirstOrDefaultAsync();

                //test special permission to execute a submit test
                if (!await base.HasGroupPermission(targetUserGroup,
                    permission => permission != null && permission.CanAssessExercises))
                {
                    await
                        Response.WriteAsync(
                            Jc.Serialize(new BasicResponse(ResponseCode.NoPermission, "no permission")));
                    return;
                }

                //submit test is EVERY time run after the release is finished... not directly by a user 

                //so only check release
                if (exerciseRelease.IsReleased == true)
                {
                    await Response.WriteAsync(Jc.Serialize(new BasicResponse(ResponseCode.ReleaseStillActive, "")));
                    return;
                }
            }
            else
            {
                //this is already done in the calling method (where the http entry point is...)

                //if it's a normal test then check working time (the submission assessment worker do not use this method so it's ensured that
                //this is run by a user (wich should have a participation)
                //var participation = _context.ExerciseReleaseWithUserAsParticipations
                //    .FirstOrDefault(p => p.ExerciseRelease.GeneratedCode == exerciseRelease.GeneratedCode && p.UserId == userId);

                //if (participation == null)
                //{
                //    return;
                //}

                //check if available working time is up

                //check if available working time is up
                //if (ReleaseHelper.IsTimeUp(participation, exerciseRelease, this))
                //{
                //    return;
                //}
            }

            //if we requested more... see tooManyTestsRequested

            var testIds = runNormalTestCommand.TestIds.Take(isSubmitTest
                    ? systemSettings.MaxNumberOfTestsWithOneRequestSubmitTestServer
                    : systemSettings.MaxNumberOfTestsWithOneRequest)
                .ToList();


            //the user as access to the tests (tests are the correct ones) because he/she got a participation for the exercise
            //and the tests are attached to the exercise
            var tests = await _context.Tests
                    .Include(p => p.TestSettings)
                    .Include(p => p.TestType)
                    .Include(p => p.AssetReferences)
                    .ThenInclude(p => p.FileReferenceTestAsset)
                    .Where(p => p.IsSubmitTest == isSubmitTest && p.ExerciseId == exerciseRelease.ExerciseId &&
                                testIds.Contains(p.Id))
                    .ToListAsync()
                ;

            var tooManyTestsRequested = tests.Count != runNormalTestCommand.TestIds.Count;


            //get test asset contents
            var testFilesIdsToRead = tests.SelectMany(p => p.AssetReferences)
                .Select(p => p.FileReferenceTestAssetId)
                .ToArray();

            var testAssetsBasePath = Files.GetUploadFilePath(UploadDirType.TestAssets);

            var testFilesDict = await Files.ReadUploadedFilesAsArray(testAssetsBasePath, testFilesIdsToRead);

            List<AllTest> allTests;

            try
            {
                allTests = tests.Select(p => AllTest.CreateFrom(p, testFilesDict)).ToList();
            }
            catch (Exception ex)
            {
                //we create this exception so it should be ok to send just the message...

                string errorMessage = ExceptionHelper.GetAndLogException(ex, "ERROR creating tests with test files");

                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, errorMessage)));
                return;
            }

            //do not include solution files here ... it might be too many data to load...
            //just find the correct version then start another query to get the solution version + files

            Solution userSolution = await _context.Solutions
                .Include(p => p.SolutionFiles)
                .Include(p => p.TestResults) //only include tests here because its a normal test (not a custom test)
                .FirstOrDefaultAsync(
                    p => p.UserId == userId && p.ExerciseReleaseId == exerciseRelease.Id && p.PLangId == plang.Id);


            if (userSolution == null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "user solution not found")));
                return;
            }

            var codeTemplate = exerciseRelease.Exercise.CodeTemplates
                    .FirstOrDefault(p => p.PLangId == userSolution.PLangId)
                ;

            if (codeTemplate == null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "codeTemplate not found")));
                return;
            }

            //make readonly files up-to-date
            var errorMsg = UserSolutionHelper.ReplaceReadonlyFiles(userSolution, codeTemplate, false, true);

            if (errorMsg != null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.ServerError, errorMsg)));
                return;
            }

            //---run the test

            bool areAllSqlCheckerTests = AreAllThirdPartySqlCheckerTests(allTests);


            if (areAllSqlCheckerTests == false)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(
                            new BasicResponse(ResponseCode.ServerError, "some tests are not sql checker tests")));
                return;
            }

            var sqlCheckerPreResults =
                await RequestSubmitTestToSqlChecker(systemSettings, userId, userSolution.MainFile.Content, allTests);


            var responseList = new List<TestAnswerFromBackend>();

            foreach (var testResult in sqlCheckerPreResults)
            {
                var test = tests.FirstOrDefault(p => p.Id == testResult.TestId);

                if (test == null)
                {
                    await
                        Response.WriteAsync(
                            Jc.Serialize(new BasicResponse(ResponseCode.ServerError,
                                "test not found for test result")));
                    return;
                }

                //get the old result
                var oldResult = userSolution.TestResults
                    .FirstOrDefault(
                        p => p.TestId == test.Id
                             && p.Solution == userSolution);
                //this is not a db query so the references are loaded

                //and store test result

                if (oldResult == null)
                {
                    //no old result, create new one
                    oldResult = createInitialTestResultFromSubmissionRequest(test, userSolution, testResult);

                    _context.TestWithSingleSolutionAsTestResult.Add(oldResult);
                }
                else
                {
                    //change old result
                    reInitTestResultFromSubmissionRequest(oldResult, userSolution, testResult);
                }

                try
                {
                    await _context.SaveChangesAsync();
                }
                catch (Exception ex)
                {
                    await base.HandleDbError(ex);
                    return;
                }

                responseList.Add(new TestAnswerFromBackend()
                {
                    TestId = oldResult.TestId,
                    ProgramExitCode = oldResult.ProgramExitCode ?? -1,
                    Passed = oldResult.Passed ?? false,
                    Protocol = new List<string>() {oldResult.Protocol},
                    HasCompiled = false,
                    RunnerVersion = oldResult.RunnerVersion,
                    CharacterLimitExceeded = oldResult.CharacterLimitExceeded,
                    CharacterLimitUsed = oldResult.CharacterLimitUsed,
                    TestResultCode = oldResult.TestResultCode ?? -1,
                    TestServerCode = -1,
                    TestServerVersion = oldResult.TestServerVersion,
                    TestServerMessage = null,
                    TimeForCompiling = oldResult.TimeForCompiling,
                    TimeForUserProgram = oldResult.TimeForCompiling,
                    TimeoutInMsUsed = oldResult.TimeoutInMsUsed,
                    CompileTimeoutInMsUsed = oldResult.CompileTimeoutInMsUsed,
                });
            }

            await Response.WriteAsync(
                Jc.Serialize(new BasicResponseWithData<List<TestAnswerFromBackend>>(ResponseCode.Ok,
                    "",
                    responseList)));
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="systemSettings"></param>
        /// <param name="userId"></param>
        /// <param name="sqlQuery"></param>
        /// <param name="allTests"></param>
        private async Task<List<SubmissionRequestAnswerBackendOnly>> RequestSubmitTestToSqlChecker(
            SystemSetting systemSettings,
            int userId, string sqlQuery, List<AllTest> allTests)
        {
            var results = new List<SubmissionRequestAnswerBackendOnly>();

            foreach (var allTest in allTests)
            {
                SqlCheckerRequestJsonPayload payload = null;

                try
                {
                    payload = Jc.Deserialize<SqlCheckerRequestJsonPayload>(allTest.ThirdPartyJsonPayload);
                }
                catch (Exception e)
                {
                    //TODO
                    Console.WriteLine(e);
                    throw;
                }

                var submitRequest = new SubmissionRequest()
                {
                    User = userId,
                    Problem = payload.ProblemId,
                    Version = payload.ProblemVersion,
                    Query = sqlQuery,
                };


                var requestPayload = new StringContent(Jc.SerializePlain(submitRequest),
                    SubmissionController.UTF8WithoutBom,
                    "application/json");

                HttpResponseMessage jsonResponse = null;
                var tokenSource = new CancellationTokenSource();

                using (var client = new HttpClient())
                {
                    //TODO MAYBE use own timeouts...
                    //do we need both?? no idea if we really need both
                    client.Timeout = new TimeSpan(0, 0, 0, 0, systemSettings.TestServerTimeoutInMs);
                    tokenSource.CancelAfter(new TimeSpan(0, 0, 0, 0, systemSettings.TestServerTimeoutInMs));

                    try
                    {
                        jsonResponse = await client.PostAsync(
                            $"{systemSettings.ThirdPartySqlCheckerBaseUrl}/submission/",
                            requestPayload, tokenSource.Token);
                    }
                    catch (Exception e)
                    {
                        //TODO
                        Console.WriteLine(e);
                        throw;
                    }
                }

                string contentResult = null;

                if (jsonResponse != null)
                    contentResult = await jsonResponse.Content.ReadAsStringAsync();


                SqlCheckerSubmissionRequestAnswer answerFromTestServer;

                tokenSource.Dispose();

                if (contentResult == null && tokenSource.IsCancellationRequested)
                {
                    //timeout...
                }

                try
                {
                    answerFromTestServer = Jc.Deserialize<SqlCheckerSubmissionRequestAnswer>(contentResult);
                }
                catch (Exception e)
                {
                    //TODO
                    Console.WriteLine(e);
                    throw;
                }

                results.Add(new SubmissionRequestAnswerBackendOnly()
                {
                    TestId = allTest.Id,
                    SubmissionId = answerFromTestServer.SubmissionId,
                    ErrorMessage = ""
                });
            }


            return results;
        }


        [HttpPost("status")]
        public async Task RequestSqlCheckerStatus([FromBody] GetSqlCheckerStatusCommandForBackend command)
        {
            if (!await base.IsLoggedIn()) return;

            int userId = GetUserId();


            if (string.IsNullOrWhiteSpace(command.GeneratedCode))
            {
                await Response.WriteAsync(Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "exercise not found")));
                return;
            }

            var systemSettings = await _context.SystemSettings.FirstOrDefaultAsync();

            if (systemSettings == null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "system settings not found")));
                return;
            }

            var participation = await _context.ExerciseReleaseWithUserAsParticipations
                .Include(p => p.ExerciseRelease)
                .ThenInclude(p => p.Exercise.CodeTemplates)
                .ThenInclude(p => p.TemplateFiles)
                .FirstOrDefaultAsync(p => p.UserId == userId
                                          //&& p.ExerciseRelease.IsReleased // is ReleaseHelper.IsTimeUp checks for is released so wo do not need to double check this
                                          && p.ExerciseRelease.GeneratedCode == command.GeneratedCode);


            if (participation?.ExerciseRelease == null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "no participation or release found")));
                return;
            }

            if (participation.ExerciseRelease.Exercise == null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "exercise not found")));
                return;
            }

            if (participation.ExerciseRelease.Exercise.IsPermanentlyLocked)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.InvalidRequest, "exercise is permanently locked")));
                return;
            }

            if (ReleaseHelper.IsTimeUp(participation, participation.ExerciseRelease, this)) return;


            if (!command.IsCustomTest && !command.IsNormalTest &&
                !command.IsSubmitTest)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.InvalidRequest, "test type not set")));
                return;
            }

            //then we can get the plang directly from the user solution...
            PLang pLangToUse = await _context.PLangs
                .FirstOrDefaultAsync(p => p.Id == command.PlangId);

            //which test kind??
            if (command.IsNormalTest)
            {
                await _RequestSqlCheckerStatus(command, userId, pLangToUse, participation.ExerciseRelease, false,
                    systemSettings);
            }
            else if (command.IsCustomTest)
            {
//                await _RunCustomTest(command, userId, pLangToUse, participation.ExerciseRelease, systemSettings);
                await Response.WriteAsync(Jc.Serialize(new BasicResponse(ResponseCode.ServerError, "not implemented")));
            }
            else if (command.IsSubmitTest)
            {
                Console.WriteLine("[WARNING] running submit tests is not supported here (yet)");

                //not used from frontend... user cannot run submit tests... maybe later for tutors?
//                await _RunNormalOrSubmitTest(command, userId, pLangToUse, participation.ExerciseRelease, true, systemSettings);
            }
        }


        private async Task _RequestSqlCheckerStatus(GetSqlCheckerStatusCommandForBackend command, int userId,
            PLang plang, ExerciseRelease exerciseRelease, bool isSubmitTest, SystemSetting systemSettings)
        {
            if (isSubmitTest)
            {
                //the user group that manages the exercise
                var targetUserGroup =
                    await _context.Exercises.Where(p => p.Id == exerciseRelease.ExerciseId)
                        .Select(p => p.UserGroupId)
                        .FirstOrDefaultAsync();

                //test special permission to execute a submit test
                if (!await base.HasGroupPermission(targetUserGroup,
                    permission => permission != null && permission.CanAssessExercises))
                {
                    await
                        Response.WriteAsync(
                            Jc.Serialize(new BasicResponse(ResponseCode.NoPermission, "no permission")));
                    return;
                }

                //submit test is EVERY time run after the release is finished... not directly by a user 

                //so only check release
                if (exerciseRelease.IsReleased == true)
                {
                    await Response.WriteAsync(Jc.Serialize(new BasicResponse(ResponseCode.ReleaseStillActive, "")));
                    return;
                }
            }
            else
            {
                //this is already done in the calling method (where the http entry point is...)

                //if it's a normal test then check working time (the submission assessment worker do not use this method so it's ensured that
                //this is run by a user (wich should have a participation)
                //var participation = _context.ExerciseReleaseWithUserAsParticipations
                //    .FirstOrDefault(p => p.ExerciseRelease.GeneratedCode == exerciseRelease.GeneratedCode && p.UserId == userId);

                //if (participation == null)
                //{
                //    return;
                //}

                //check if available working time is up

                //check if available working time is up
                //if (ReleaseHelper.IsTimeUp(participation, exerciseRelease, this))
                //{
                //    return;
                //}
            }


            //we don't need the tests here just query the submission state from the sql checker

            if (command.TestIds.Count != 1)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.InvalidRequest,
                            $"we support only one test at a time")));
                return;
            }

            if (command.SubmissionIds.Count != 1)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.InvalidRequest,
                            $"only 1 submission is supported (for now)")));
                return;
            }

            var firstTestId = command.TestIds.First();


            var tests = await _context.Tests
                    .Include(p => p.TestSettings)
                    .Include(p => p.TestType)
                    .Where(p => p.IsSubmitTest == isSubmitTest && p.ExerciseId == exerciseRelease.ExerciseId &&
                                p.Id == firstTestId)
                    .ToListAsync()
                ;


            bool areAllSqlCheckerTests = AreAllThirdPartySqlCheckerTests(tests);


            if (areAllSqlCheckerTests == false)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(
                            new BasicResponse(ResponseCode.ServerError, "some tests are not sql checker tests")));
                return;
            }

            //ensure the user gets only results for him/her not from another user...

            //do not include solution files here ... it might be too many data to load...
            //just find the correct version then start another query to get the solution version + files

            Solution userSolution = await _context.Solutions
                .Include(p => p.SolutionFiles)
                .Include(p => p.TestResults) //only include tests here because its a normal test (not a custom test)
                .FirstOrDefaultAsync(
                    p => p.UserId == userId && p.ExerciseReleaseId == exerciseRelease.Id && p.PLangId == plang.Id);


            if (userSolution == null)
            {
                await
                    Response.WriteAsync(
                        Jc.Serialize(new BasicResponse(ResponseCode.NotFound, "user solution not found")));
                return;
            }

            var checkedSubmissionIds = new List<Tuple<int, int>>();
            var testResults = new List<TestWithSingleSolutionAsTestResult>();

            foreach (var test in tests)
            {
                //get the old result
                var oldResult = userSolution.TestResults
                    .FirstOrDefault(
                        p => p.TestId == test.Id
                             && p.Solution == userSolution);

                if (oldResult == null) continue;

                //not double get results...
//                if (IsPendingSqlCheckerSubmission(oldResult) == false ||
//                    oldResult.ProgramExitCode.HasValue == false) continue;

                checkedSubmissionIds.Add(new Tuple<int, int>(oldResult.TestId, oldResult.ProgramExitCode ?? -1));
                testResults.Add(oldResult);
            }


            //--- get test test result

            var sqlCheckerResponses = await RequestResultFromSqlChecker(systemSettings, checkedSubmissionIds);


            var response = new List<GetSqlCheckerStatusResponseFromBackend>();
            //save the result...

            //only save if we got some useful result...
            bool needToSaveResult = false;

            foreach (var oldResult in testResults)
            {
                if (oldResult.ProgramExitCode == null) continue;

                var sqlCheckerResponse = sqlCheckerResponses.Find(p => p.Item1.TestId == oldResult.TestId);

                if (sqlCheckerResponse.Item1.SqlCheckerSubmissionInfo == null)
                {
                    //we could not get any result...

                    if (sqlCheckerResponse.Item2 == null)
                    {
                        //also not error msg... so just the sql checker hasn't finished yet 
                        response.Add(new GetSqlCheckerStatusResponseFromBackend()
                        {
                            TestId = oldResult.TestId,
                            testResult = null
                        });
                    }
                    else
                    {
                        //some error...
                        if (AppConfiguration.IsDebugMode)
                        {
                            await
                                Response.WriteAsync(
                                    Jc.Serialize(new BasicResponse(ResponseCode.ServerError, "some error communicating with (or during) the sql checker: " + sqlCheckerResponse.Item2)));
                            return;
                        }
                        else
                        {
                            await
                                Response.WriteAsync(
                                    Jc.Serialize(new BasicResponse(ResponseCode.ServerError, "some error communicating with (or during) the sql checker")));
                            return;
                        }
                        
                    }

                    continue;
                }

                needToSaveResult = true;
                finishTestResult(oldResult, sqlCheckerResponse.Item1.SqlCheckerSubmissionInfo);

                response.Add(new GetSqlCheckerStatusResponseFromBackend()
                {
                    TestId = oldResult.TestId,
                    testResult = new TestAnswerFromBackend()
                    {
                        TestId = oldResult.TestId,
                        ProgramExitCode = oldResult.ProgramExitCode ?? -1,
                        Passed = oldResult.Passed ?? false,
                        Protocol = new List<string>() {oldResult.Protocol},
                        HasCompiled = false,
                        RunnerVersion = oldResult.RunnerVersion,
                        CharacterLimitExceeded = oldResult.CharacterLimitExceeded,
                        CharacterLimitUsed = oldResult.CharacterLimitUsed,
                        TestResultCode = oldResult.TestResultCode ?? -1,
                        TestServerCode = -1,
                        TestServerVersion = oldResult.TestServerVersion,
                        TestServerMessage = null,
                        TimeForCompiling = oldResult.TimeForCompiling,
                        TimeForUserProgram = oldResult.TimeForCompiling,
                        TimeoutInMsUsed = oldResult.TimeoutInMsUsed,
                        CompileTimeoutInMsUsed = oldResult.CompileTimeoutInMsUsed,
                    }
                });
            }

            if (needToSaveResult)
            {
                try
                {
                    await _context.SaveChangesAsync();
                }
                catch (Exception ex)
                {
                    await base.HandleDbError(ex);
                    return;
                }
            }


            await Response.WriteAsync(
                Jc.Serialize(new BasicResponseWithData<List<GetSqlCheckerStatusResponseFromBackend>>(ResponseCode.Ok,
                    "",
                    response)));
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="systemSettings"></param>
        /// <param name="userId"></param>
        /// <param name="testIdAndSubmissionIds"></param>
        /// <returns>[0] null or the info, [1] null or the error message</returns>
        private async Task<List<Tuple<GetSqlCheckerStatusResponseBackendOnly, string>>> RequestResultFromSqlChecker(
            SystemSetting systemSettings, List<Tuple<int, int>> testIdAndSubmissionIds)
        {
            var results = new List<Tuple<GetSqlCheckerStatusResponseBackendOnly, string>>();
            

            foreach (var submissionId in testIdAndSubmissionIds)
            {
                var errorResponseObj = new GetSqlCheckerStatusResponseBackendOnly()
                {
                    TestId = submissionId.Item1,
                    SqlCheckerSubmissionInfo = null
                };
                
                HttpResponseMessage jsonResponse = null;
                var tokenSource = new CancellationTokenSource();

                using (var client = new HttpClient())
                {
                    //TODO MAYBE use own timeouts...
                    //do we need both?? no idea if we really need both
                    client.Timeout = new TimeSpan(0, 0, 0, 0, systemSettings.TestServerTimeoutInMs);
                    tokenSource.CancelAfter(new TimeSpan(0, 0, 0, 0, systemSettings.TestServerTimeoutInMs));

                    try
                    {
                        jsonResponse =
                            await client.GetAsync(
                                $"{systemSettings.ThirdPartySqlCheckerBaseUrl}submission_info/?submissionId={submissionId.Item2}",
                                tokenSource.Token);
                    }
                    catch (Exception e)
                    {
                        string errorMsg = ExceptionHelper.GetAndLogException(e, "sql checker request");
                        results.Add(new Tuple<GetSqlCheckerStatusResponseBackendOnly, string>(errorResponseObj, errorMsg));
                        continue;
                    }
                }

                string contentResult = null;

                if (jsonResponse != null)
                    contentResult = await jsonResponse.Content.ReadAsStringAsync();


                SqlCheckerSubmissionInfo answerFromTestServer;

                tokenSource.Dispose();

                if (contentResult == null && tokenSource.IsCancellationRequested)
                {
                    string errorMsg = "Timeout getting sql checker response";
                    results.Add(new Tuple<GetSqlCheckerStatusResponseBackendOnly, string>(errorResponseObj, errorMsg));
                    continue;
                }

                //we get this if the result is not ready yet...
                if (jsonResponse.StatusCode == HttpStatusCode.NotFound)
                {
                    results.Add(new Tuple<GetSqlCheckerStatusResponseBackendOnly, string>(errorResponseObj, null));
                }
                else
                {
                    answerFromTestServer = Jc.Deserialize<SqlCheckerSubmissionInfo>(contentResult);

                    if (answerFromTestServer == null)
                    {
                        //could not deserialize...
                        string errorMsg = "could not deserialize sql checker response";
                        results.Add(new Tuple<GetSqlCheckerStatusResponseBackendOnly, string>(errorResponseObj, errorMsg));
                        continue;
                    }

                    var response = new GetSqlCheckerStatusResponseBackendOnly()
                    {
                        TestId = submissionId.Item1,
                        SqlCheckerSubmissionInfo = answerFromTestServer
                    };
                    
                    results.Add(new Tuple<GetSqlCheckerStatusResponseBackendOnly, string>(response, null));
                }

                
            }

            return results;
        }


        //---------- helper functions ------------

        private bool AreAllThirdPartySqlCheckerTests(List<AllTest> tests)
        {
            return tests.All(p => p.TestType.InternalName == Constants.IntegrationSqlCheckerTestInternalName);
        }

        private bool AreAllThirdPartySqlCheckerTests(List<Test> tests)
        {
            return tests.All(p => p.TestType.InternalName == Constants.IntegrationSqlCheckerTestInternalName);
        }

        private bool IsPendingSqlCheckerSubmission(TestWithSingleSolutionAsTestResult testResult)
        {
            return testResult.Passed == null && testResult.ProgramExitCode.HasValue;
        }

        private TestWithSingleSolutionAsTestResult createInitialTestResultFromSubmissionRequest(Test test,
            Solution userSolution,
            SubmissionRequestAnswerBackendOnly submissionRequestAnswerFromChecker)
        {
            var initialTestResult = new TestWithSingleSolutionAsTestResult()
            {
                Solution = userSolution,
                Test = test,
                Passed = null,
                Protocol = "",
                HasCompiled = null,
                ProgramExitCode = submissionRequestAnswerFromChecker.SubmissionId,
                TestResultCode = null,
                RunnerVersion = "-1",
                TestServerVersion = "-1",
                TimeForCompiling = null,
                TimeForUserProgram = null,
                TimeoutInMsUsed = null,
                CompileTimeoutInMsUsed = null,
                CharacterLimitExceeded = false,
                CharacterLimitUsed = null,
            };


            return initialTestResult;
        }

        private void reInitTestResultFromSubmissionRequest(TestWithSingleSolutionAsTestResult oldTest,
            Solution userSolution,
            SubmissionRequestAnswerBackendOnly submissionRequestAnswerFromChecker)
        {
            oldTest.Passed = null;
            oldTest.Protocol = "";
            oldTest.HasCompiled = null;
            oldTest.ProgramExitCode = submissionRequestAnswerFromChecker.SubmissionId;
            oldTest.TestResultCode = null;
            oldTest.RunnerVersion = "-1";
            oldTest.TestServerVersion = "-1";
            oldTest.TimeForCompiling = null;
            oldTest.TimeForUserProgram = null;
            oldTest.TimeoutInMsUsed = null;
            oldTest.CompileTimeoutInMsUsed = null;
            oldTest.CharacterLimitExceeded = false;
            oldTest.CharacterLimitUsed = null;
        }

        private void finishTestResult(TestWithSingleSolutionAsTestResult oldResult,
            SqlCheckerSubmissionInfo responseFromSqlChecker)
        {
            oldResult.Passed = (responseFromSqlChecker.Result == SubmissionInfoResult.RESULT_EQUIVALENT ||
                                responseFromSqlChecker.Result == SubmissionInfoResult.EQUIVALENT);
            oldResult.Protocol = Jc.Serialize(responseFromSqlChecker);
            oldResult.HasCompiled = true;
            oldResult.ProgramExitCode = responseFromSqlChecker.SubmissionId; //already set
            oldResult.TestResultCode = 0;

//            oldResult.RunnerVersion = testResult.RunnerVersion;
//            oldResult.TestServerVersion = testResult.TestServerVersion;
//            oldResult.TimeForCompiling = testResult.TimeForCompiling;
//            oldResult.TimeForUserProgram = testResult.TimeForUserProgram;
//            oldResult.TimeoutInMsUsed = testResult.TimeoutInMsUsed;
//            oldResult.CompileTimeoutInMsUsed = testResult.CompileTimeoutInMsUsed;
//            oldResult.CharacterLimitExceeded = testResult.CharacterLimitExceeded;
//            oldResult.CharacterLimitUsed = testResult.CharacterLimitUsed;
        }
    }

    /// <summary>
    /// a test request/command for backend
    ///
    /// we only support one element in <see cref="RunTestCommandForBackend.TestIds"/> here
    /// </summary>
    public class GetSqlCheckerStatusCommandForBackend : RunTestCommandForBackend
    {
        public List<int> SubmissionIds { get; set; }
    }


    public class GetSqlCheckerStatusResponseBackendOnly
    {
        public int TestId { get; set; }
        public SqlCheckerSubmissionInfo SqlCheckerSubmissionInfo { get; set; }
    }

    public class GetSqlCheckerStatusResponseFromBackend
    {
        public int TestId { get; set; }
        /// <summary>
        /// the protocol contains SqlCheckerSubmissionInfo
        /// CAN BE NULL
        /// </summary>
        public TestAnswerFromBackend testResult { get; set; }

//        public SqlCheckerSubmissionInfo SqlCheckerSubmissionInfo { get; set; }
    }


    //see
    //https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/webapp/src/main/java/servlets/SubmissionInfoServlet.java
    //https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/webapp/src/main/java/servlets/SubmissionServlet.java
    //TODO https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/webapp/src/main/java/servlets/SubmissionResultServlet.java

    //TODO some names???

    //TODO windows/linux newlines??
    //see also https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/ostepu-integration/check_query_at_upload.php


    /// <summary>
    /// represents a request/submission for the sql checker
    ///
    /// normally we get the submission id back
    ///
    /// see https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/webapp/src/main/java/servlets/SubmissionServlet.java
    /// </summary>
    public class SubmissionRequest
    {
        /// <summary>
        /// the sql query
        /// </summary>
        public string Query { get; set; }

        /// <summary>
        /// currently always 0!
        /// ID of the user who submitted query
        /// </summary>
        public int User { get; set; }

        /// <summary>
        /// the problem id
        /// ID of the exercise the submitted query is for
        /// </summary>
        public int Problem { get; set; }

        /// <summary>
        /// ID of the version of the exercise the query is for
        /// </summary>
        public int Version { get; set; }

        public SubmissionRequest()
        {
        }
    }

    public class SqlCheckerSubmissionRequestAnswer
    {
        public int SubmissionId { get; set; }
    }


    public class SubmissionRequestAnswerBackendOnly
    {
        public int TestId { get; set; }

        /// <summary>
        /// null (on error) or the submission id
        /// </summary>
        public int? SubmissionId { get; set; }

        /// <summary>
        /// the error message caught while submitting to the sql checker (or receiving the submission id)
        /// </summary>
        public string ErrorMessage { get; set; }
    }

    /// <summary>
    /// information about the submission can be
    /// the result (200, json) or if not ready get just a text (404, text)
    ///
    /// see https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/webapp/src/main/java/servlets/SubmissionInfoServlet.java
    /// see for data type https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/webapp/src/main/java/helpers/SubmissionInfoDatatype.java
    /// </summary>
    public class SqlCheckerSubmissionInfo
    {
        public int SubmissionId { get; set; }
        public string SubmissionSql { get; set; }

        public int ProblemId { get; set; }
        public int ProblemVersion { get; set; }

        public bool TimeoutQREC { get; set; }

        public bool TimeoutQEC { get; set; }

        /// <summary>
        /// see https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/worker/src/main/java/helpers/OperatorDatatype.java
        /// </summary>
        // see https://bytefish.de/blog/enums_json_net/
        [JsonConverter(typeof(StringEnumConverter))]
        public SubmissionInfoResult Result { get; set; }


        /// <summary>
        /// can be explicitly null
        /// </summary>
        public string CheckerMessage { get; set; }
    }


    /// <summary>
    /// see https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/worker/src/main/java/helpers/OperatorDatatype.java
    /// TODO these are not the real values...
    /// see https://gitlab.informatik.uni-halle.de/sql/sql-checker/blob/master/worker/src/main/sql/init_problem_db/problem-database.sql > CREATE VIEW result ...
    /// NOT the check_result enum !!
    /// </summary>
    public enum SubmissionInfoResult
    {
        /// <summary>
        /// manually_assessed
        /// </summary>
        EQUIVALENT_BY_CONTROLLER,

        /// <summary>
        /// manually_assessed
        /// </summary>
        NOT_EQUIVALENT_BY_CONTROLLER,

        /// <summary>
        /// syntactically_correct is unknown
        /// </summary>
        ERROR,

        /// <summary>
        /// syntactically_correct is false and check_queryresult_checker is null
        /// </summary>
        NOT_PARSABLE,

        /// <summary>
        /// syntactically_correct is false and check_queryresult_checker is set
        /// </summary>
        NOT_EXECUTABLE,

        /// <summary>
        /// result is equivalent by some internal checker
        /// </summary>
        EQUIVALENT,

        /// <summary>
        /// result is not equivalent by some internal checker
        /// </summary>
        NOT_EQUIVALENT,

        /// <summary>
        /// result is equivalent by some internal checker
        /// </summary>
        RESULT_EQUIVALENT,

        /// <summary>
        /// this can happen when we have a timeout or unknown in at least one qrec state
        /// </summary>
        UNKNOWN
    }
}
