using System;

namespace ClientServer.Helpers
{
    public static class ExceptionHelper
    {
        /// <summary>
        /// a full stack trace might be too much for debug mode... just return exception message and first inner exception
        /// the full stack trace should be logged to the console
        ///
        /// this should be only called if <see cref="AppConfiguration.IsDebugMode"/> is true! else a generic error string should be returned e.g. only db server error
        /// </summary>
        /// <param name="ex"></param>
        /// <param name="errorDescription">some description e.g. db error or the file name to be logged to the console</param>
        /// <returns></returns>
        public static string GetAndLogException(Exception ex, string errorDescription)
        {

            var exceptionGuid = Guid.NewGuid().ToString();
            Console.WriteLine($"[{errorDescription}] - {exceptionGuid} - " + ex);
            
            return  "[" + errorDescription + "] - guid: " + exceptionGuid + " - " + ex.Message + " | " + (ex.InnerException != null ? ex.InnerException.Message : "");
        }
    }
}
