﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using ClientServer.Db;
using ClientServer.Models.Exercises.Release;
using ClientServer.Models.Exercises.Solution;
using ClientServer.Models.Exercises.Tests;
using ClientServer.Models.Users;
using ClientServer.Models.Interfaces;

namespace ClientServer.Models.Exercises
{
    /// <summary>
    /// an exercise to solve
    /// </summary>
    public class Exercise
    {

        public Exercise()
        {
            Releases = new List<ExerciseRelease>();
            Tests = new List<Test>();
        }

        public int Id { get; set; }
        /// <summary>
        /// the name
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string DisplayName { get; set; }
        /// <summary>
        /// a short description
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string ShortDescription { get; set; }

        /// <summary>
        /// true: user can create own/custom test cases, false: not
        /// </summary>
        public bool CanUserCreateCustomTests { get; set; }

        /// <summary>
        /// true: the user can add new tabs (new files) to the solution
        /// </summary>
        public bool CanUserCreateFiles { get; set; }

        /// <summary>
        /// true: only visible to the creator/owner of the exercise in the groups exercise overview
        /// AND only hte owner can create/update... exercise, 
        /// false: do not show to others in group ...
        /// IF the own is removed then the group then????
        /// </summary>
        public bool IsOnlyVisibleToOwner { get; set; }

        /// <summary>
        /// exercise and all dependent entities should be readonly (no edit/no delete)! (recursively / transitively) 
        /// this is used as backup exercise e.g. for exams
        /// one can only copy this exercise and modify the copy
        ///
        /// so the backend must ensure that all dependent entities are also locked in the controllers)
        /// </summary>
        public bool IsPermanentlyLocked { get; set; }

        /// <summary>
        /// notes for the exercise (e.g. to change something in the future...)
        /// </summary>
        [MaxLength(YapexDbContext.NotesMaxStringLength)]
        public string Note { get; set; }


        /// <summary>
        /// the date time the exercise was last modified
        /// </summary>
        public DateTime LastUpdatedAt { get; set; }

        public DateTime CreatedAt { get; set; }


        /// <summary>
        /// the related <see cref="MetaData"/> object
        /// </summary>
        public MetaData MetaData { get; set; }

        /// <summary>
        /// the related <see cref="CodeTemplate"/> s
        /// </summary>
        public List<CodeTemplate> CodeTemplates { get; set; }


        /// <summary>
        /// the <see cref="Description"/> for this <see cref="Exercise"/>
        /// </summary>
        public ExerciseDescription Description { get; set; }

        /// <summary>
        /// all related <see cref="ExerciseRelease"/> s for this <see cref="Exercise"/>
        /// </summary>
        public List<ExerciseRelease> Releases { get; set; }

        /// <summary>
        /// all related <see cref="Test"/> s for this <see cref="Exercise"/>
        /// </summary>
        public List<Test> Tests { get; set; }

        /// <summary>
        /// the related custom test case settings
        /// </summary>
        public DefaultCustomTestSettings DefaultCustomTestSettings { get; set; }



        public int? UserId { get; set; }
        /// <summary>
        /// the creator of this <see cref="Exercise"/>, can be null
        /// then the exercise should be managed by the user group
        /// </summary>
        public User User { get; set; }


        public int UserGroupId { get; set; }
        /// <summary>
        /// the user group the exercise belongs to
        /// </summary>
        public UserGroup UserGroup { get; set; }

    }
}
