﻿using System;
using System.Collections.Generic;
using ClientServer.Models.Exercises.Release;
using ClientServer.Models.Exercises.Tests;
using ClientServer.Models.Users;

namespace ClientServer.Models.Exercises.Solution
{
    /// <summary>
    /// a class to store which user has entered which code (which user can access which exercise)
    /// and stores the solution for an exercise
    /// </summary>
    public class ExerciseReleaseWithUserAsParticipation
    {

        public ExerciseReleaseWithUserAsParticipation()
        {
            Solutions = new List<Solution>();
            CustomTests = new List<CustomTest>();
        }

        //pk
        public int ExerciseReleaseId { get; set; }
        /// <summary>
        /// the connected <see cref="ExerciseRelease"/>
        /// </summary>
        public ExerciseRelease ExerciseRelease { get; set; }

        //pk
        public int UserId { get; set; }
        /// <summary>
        /// the connected <see cref="User"/>
        /// </summary>
        public User User { get; set; }


        /// <summary>
        /// true: the student wants that his/her solution is not counted (for whatever), false: not
        /// can only be set once by the student
        ///
        /// this is not in the assessment/solution because the user can later solve the solution in different
        /// p langs (user can choose) but this should count for all p langs
        /// default: false --> should count
        /// </summary>
        public bool ShouldNotCount { get; set; }

        /// <summary>
        /// use shadow property to determine the date time when the user started working in the solution <see cref="ExerciseRelease.AvailableWorkingTimeInMinutes"/>
        /// because it's a shadow property we do not need to set this manually... its set by the entity framework during save changes
        /// </summary>
        public DateTime CreatedAt { get; set; }

        public DateTime LastUpdatedAt { get; set; }

        /// <summary>
        /// the tests the user has created for this exercise (release)
        /// </summary>
        public List<CustomTest> CustomTests { get; set; }

        /// <summary>
        /// list with solutions (for every p lang at most one)
        /// TODO releases currently only have 1 plang...
        /// </summary>
        public List<Solution> Solutions { get; set; }


        public int LastEditedPLangId { get; set; }
        /// <summary>
        /// the last edited <see cref="PLang"/> (selected in frontend)
        /// </summary>
        public PLang LastEditedPLang { get; set; }

        /// <summary>
        /// just a flag for the frontend to lock the editor and hide the left working time
        /// so it feels for the user as if he 'send' a submission (displayed as checkbox)
        /// </summary>
        public bool LockSolutionsFlag { get; set; }

    }
}
