﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Threading.Tasks;
using ClientServer.Db;
using ClientServer.Helpers;
using ClientServer.Models.Exercises.AfterSolutions;

namespace ClientServer.Models.Exercises.Solution
{
    /// <summary>
    /// a code solution for an <see cref="Exercise"/>+
    /// one solution version per plang & user
    /// </summary>
    public class Solution 
    {

        public Solution()
        {
            TestResults = new List<TestWithSingleSolutionAsTestResult>();
            CustomTestResults = new List<CustomTestWithSingleSolutionAsTestResult>();
//            SolutionFiles = new List<SolutionFile>();
        }
        
        //pk
        public int PLangId { get; set; }

        
        /// <summary>
        /// the programming language connected to this solution
        /// the release also stores the p language but can be null (if released with all languages)
        /// </summary>
        public PLang PLang { get; set; }


//        /// <summary>
//        /// this is NOT a Foreign Key (FK)! this marks the given solution version id as the main one (that sould be automatically assessed)
//        /// if this id is invalid the latest solution version is taken!
//        /// currently this is not used and ALWAYS 0!
//        /// 
//        /// this should be later used so that the user can set the right version that should be assessed
//        /// </summary>
//        public int MainSolutionVersionid { get; set; }


//        /// <summary>
//        /// the solution versions
//        /// </summary>
//        public List<SolutionVersion> SolutionVersions { get; set; }


        //pk
        public int UserId { get; set; }
        //pk
        public int ExerciseReleaseId { get; set; }

        /// <summary>
        /// the connected <see cref="ExerciseReleaseWithUserAsParticipation"/>
        /// </summary>
        public ExerciseReleaseWithUserAsParticipation ExerciseReleaseWithUserAsParticipation { get; set; }

        /// <summary>
        /// the <see cref="SolutionAssessment"/> for this solution
        /// can be null if we have not started the assessment yet
        /// </summary>
        public SolutionAssessment Assessment { get; set; }
        
        
        //---from old solution version (we now have only one solution (version))
        
        /// <summary>
        /// the datetime where the solution was created
        /// </summary>
        public DateTime CreatedAt { get; set; }
        /// <summary>
        /// the datetime when the solution (code was updated)
        /// the release participation is same or newer (because tests are connected to the participation)
        /// </summary>
        public DateTime LastUpdatedAt { get; set; }
        
        public int? MainFileId { get; set; }
        /// <summary>
        /// the main entry point for the program (e.g. file with the public static void main...)
        /// </summary>
        public SolutionFile MainFile { get; set; }
        
        /// <summary>
        /// a note about the version
        /// from the user
        /// NOT USED YET
        /// </summary>
        [MaxLength(YapexDbContext.NotesMaxStringLength)]
        public string Note { get; set; }

        /// <summary>
        /// the connected files for this solution version
        /// NOTE that these contain contain copies of readonly files from the exercise
        /// DO replace readonly files in this list with the version from the exercise (because this is always up to date)
        /// USE the helper <see cref="UserSolutionHelper"/> to replace the readonly files
        /// </summary>
        public List<SolutionFile> SolutionFiles { get; set; }
        
        /// <summary>
        /// the connected <see cref="TestWithSingleSolutionAsTestResult"/>  
        /// </summary>
        public List<TestWithSingleSolutionAsTestResult> TestResults { get; set; }

        /// <summary>
        /// the connected <see cref="CustomTestWithSingleSolutionAsTestResult"/>
        /// </summary>
        public List<CustomTestWithSingleSolutionAsTestResult> CustomTestResults { get; set; }

        /// <summary>
        /// the after solution if any
        /// </summary>
        public AfterSolution AfterSolution { get; set; }
        
        /// <summary>
        /// the last ip address when a user created or changed the solution
        /// can be null if not captured (before we introduced this prop)
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string LastEditingIpAddress { get; set; }
    }
}
