﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Threading.Tasks;
using ClientServer.Db;

namespace ClientServer.Models.Exercises.Solution
{
    /// <summary>
    /// an assessment for a <see cref="Solution"/>
    /// </summary>
    public class SolutionAssessment
    {
        public int Id { get; set; }

        /// <summary>
        /// the summed test points from the submit tests
        /// 
        /// can be null if not assessed
        /// </summary>
        public int? SubmitTestPoints { get; set; }
        /// <summary>
        /// the summed max reachable submit test points 
        /// this is even set for dummy assessments
        /// </summary>
        public int MaxSubmitTestPoints { get; set; }
        /// <summary>
        /// the summed points from the normal test
        /// 
        /// can be null if not assessed
        /// </summary>
        public int? NormalTestPoints { get; set; }
        /// <summary>
        /// the summed max reachable normal test points 
        /// this is even set for dummy assessments
        /// </summary>
        public int MaxNormalTestPoints { get; set; }
        /// <summary>
        /// optional the manual given poitns (by the tutor)
        /// </summary>
        public int? ManualPoints { get; set; }

        /// <summary>
        /// null for no error or the empty string OR
        /// the error message from the test server or some other source
        /// </summary>
        [MaxLength(YapexDbContext.TestContentMaxStringLength)]
        public string LastAssessmentErrorMessage { get; set; }

        /// <summary>
        /// the note from the tutor
        /// </summary>
        [MaxLength(YapexDbContext.NotesMaxStringLength)]
        public string NoteForOtherTutors { get; set; }

        /// <summary>
        /// the note for the student why he/she gets the manual points (feedback)
        /// </summary>
        [MaxLength(YapexDbContext.NotesMaxStringLength)]
        public string FeedbackForStudent { get; set; }

        //fks of solution
        public int PLangId { get; set; }
        public int UserId { get; set; }
        public int ExerciseReleaseId { get; set; }
        /// <summary>
        /// the connected <see cref="Solution"/>
        /// </summary>
        public Solution Solution { get; set; }


//        public int? SolutionVersionId { get; set; }
//        /// <summary>
//        /// the <see cref="SolutionVersion"/> used for the assessment
//        /// always the latest version is used (normally) or null if the user did not submit something
//        /// </summary>
//        public SolutionVersion SolutionVersion { get; set; }
        

    }
}
