﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Threading.Tasks;
using ClientServer.Db;

namespace ClientServer.Models.Users
{
    /// <summary>
    /// only accept the user request if the given token matches this token and this token is not expired...
    /// </summary>
    public class AuthToken
    {
 
        public int UserId { get; set; }
        /// <summary>
        /// the connected user
        /// </summary>
        public User User { get; set; }

        /// <summary>
        /// the random secret that was used to generate the auth token
        /// this is appended to the server secret to get a unique key for every user
        /// CURRENTLY NOT USED
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string RandomSecret { get; set; }

        /// <summary>
        /// a guid
        /// no more jwt token because if we rapidly change the tokens then the frontend and backend tokens get out of sync ...
        /// also if we extend the token lifetime we don't change the content (jwt token would change because other expiration date ...)
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string UserAuthToken { get; set; }

        /// <summary>
        /// the crsf token which is needed for any request (after login)
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string CsrfToken { get; set; }


        /// <summary>
        /// the dat + time when the token gets invalid
        /// </summary>
        public DateTime ExpirationDateTime { get; set; }
    }
}
