﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using ClientServer.Db;
using ClientServer.Models.Exercises;
using ClientServer.Models.Exercises.Release;
using ClientServer.Models.Exercises.Solution;
using ClientServer.Models.Users.Settings;
using ClientServer.Models.CustomProjects;

namespace ClientServer.Models.Users
{
    /// <summary>
    /// represents an user
    /// </summary>
    public class User
    {

        public User()
        {
            UserWithUserGroups = new List<UserWithUserGroup>();
            ExerciseReleaseWithUserAsParticipations =new List<ExerciseReleaseWithUserAsParticipation>();
            Exercises = new List<Exercise>();
        }

        public int Id { get; set; }

        /// <summary>
        /// the letter code (e.g. ahbxf)
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string Token { get; set; }

        /// <summary>
        /// the password 
        /// CAN BE NULL if we wait for the user to enter a password after he was redirected for the first time from ostepu (external login)
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string Password { get; set; }

        /// <summary>
        /// the first name
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string FirstName { get; set; }

        /// <summary>
        /// the last name
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string LastName { get; set; }

        /// <summary>
        /// the email
        /// </summary>
        [MaxLength(YapexDbContext.DefaultMaxStringLength)]
        public string Email { get; set; }

        /// <summary>
        /// true: the user is active (normal state),
        /// false: the user was not verified and cannot login yet (an admin needs to active this account)
        /// </summary>
        public bool IsActivated { get; set; }

        /// <summary>
        /// the date time the user last login
        /// </summary>
        public DateTime LastLoginAt { get; set; }

        /// <summary>
        /// the date time the user was created
        /// (automatically set via shadow property)
        /// </summary>
        public DateTime CreatedAt { get; set; }
        
        /// <summary>
        /// (automatically set via shadow property)
        /// </summary>
        public DateTime LastUpdatedAt { get; set; }

        //foreign //


        /// <summary>
        /// all related <see cref="UserWithUserGroup"/> s (this <see cref="User"/> can belong to multiple <see cref="UserWithUserGroup"/> s)
        /// </summary>
        public List<UserWithUserGroup> UserWithUserGroups { get; set; }



        /// <summary>
        /// the user settings (is optional)
        /// </summary>
        public UserSetting UserSettings { get; set; }


        /// <summary>
        /// the <see cref="Exercise"/> s where this user was the creator / is the owner
        /// </summary>
        public List<Exercise> Exercises { get; set; }

        /// <summary>
        /// the list of all custom projects
        /// </summary>
        public List<CustomProject> CustomProjects { get; set; }

        public int? SystemRoleId { get; set; }
        /// <summary>
        /// the optional connected <see cref="SystemRole"/>
        /// </summary>
        public SystemRole SystemRole { get; set; }

        /// <summary>
        /// the optional connected <see cref="ExternalUser"/>
        /// </summary>
        public ExternalUser ExternalUser { get; set; }


        /// <summary>
        /// the <see cref="ExerciseRelease"/> s exercise solutions
        /// </summary>
        public List<ExerciseReleaseWithUserAsParticipation> ExerciseReleaseWithUserAsParticipations { get; set; }
    }
}
