﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using ClientServer.Controllers.Core.Testing.ThirdParty;
using ClientServer.Db;
using ClientServer.Helpers;
using ClientServer.Schedulers;
using ClientServer.Workers;
using Microsoft.AspNetCore.Hosting;
using Microsoft.EntityFrameworkCore;

namespace ClientServer
{
    public class Program
    {
        public static void Main(string[] args)
        {

            var root = Directory.GetCurrentDirectory();
            Constants.rootDir = root;

            var builder = new WebHostBuilder()
                .UseContentRoot(root)
                .UseKestrel(opts => { 
                    //opts.MaxRequestBufferSize = 1048576;
                })
                .UseIISIntegration()
                .UseStartup<Startup>()
                ;


            var host = builder
                .UseUrls("http://*:" + Constants.Port) //cannot be used as dynamic because appsettings is read after this line...
                //http://localhost:5000 //see https://github.com/aspnet/KestrelHttpServer/issues/639
                .Build();

            
            if (AppConfiguration.IsSlave == false)
            {
                ReleaseWorker.NextRun(); //this is sync (because fast)
                ReleaseWorker.Start();

                //this is async
                SubmissionAssessmentWorker.StartNextNewThread(-1, true); //rerun the ones we don't finished e.g. because we shut down the server
                SubmissionAssessmentWorker.Start();
                
                //delete all files that are (still) in the file system but not in the db
                FileWorker.NextRun(); //this is sync (because fast)
                FileWorker.Start();
                
                TestServerTraceLogWorker.NextRun(); //this is sync
                TestServerTraceLogWorker.Start();
            }

            host.Run();
        }
    }
}
