<?php

//-- cpu usage in % but without the sign
//from http://php.net/manual/de/function.sys-getloadavg.php

function _getServerLoadLinuxData()
{
    if (is_readable("/proc/stat")) {
        $stats = @file_get_contents("/proc/stat");

        if ($stats !== false) {
            // Remove double spaces to make it easier to extract values with explode()
            $stats = preg_replace("/[[:blank:]]+/", " ", $stats);

            // Separate lines
            $stats = str_replace(array("\r\n", "\n\r", "\r"), "\n", $stats);
            $stats = explode("\n", $stats);

            // Separate values and find line for main CPU load
            foreach ($stats as $statLine) {
                $statLineData = explode(" ", trim($statLine));

                // Found!
                if
                (
                    (count($statLineData) >= 5) &&
                    ($statLineData[0] == "cpu")
                ) {
                    return array(
                        $statLineData[1],
                        $statLineData[2],
                        $statLineData[3],
                        $statLineData[4],
                    );
                }
            }
        }
    }

    return null;
}

// Returns server load in percent (just number, without percent sign)
function getServerLoad()
{
    $load = null;

    if (strtoupper(substr(PHP_OS, 0, 3)) === 'DAR') {
        //|| strtoupper(substr(PHP_OS, 0, 3)) === 'LINUX') {

        //mac os sierra gives CPU usage: 5.53% user, 10.76% sys, 83.69% idle
        //we get 5.53% 10.76% 83.69%
        $cpuUsageLine = shell_exec('top -l 1 -n 0 | sed -n 4p | awk \'{print $3, $5, $7}\'');

        $percentages = explode(' ', $cpuUsageLine);
        $usrUsage = $percentages[0];
        $sysUsage = $percentages[1];
        $idleUsage = $percentages[2];
        $idleUsageValue = substr($idleUsage, 0, strlen($idleUsage) - 2); //-2 because the last is \n

        $load = floatval($idleUsageValue);

        //$load = sys_getloadavg();
        return 100 - $load;
    } else if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
        $cmd = "wmic cpu get loadpercentage /all";
        @exec($cmd, $output);

        if ($output) {
            foreach ($output as $line) {
                if ($line && preg_match("/^[0-9]+\$/", $line)) {
                    $load = $line;
                    break;
                }
            }
        }
    } else { //assume linux

        if (is_readable("/proc/stat")) {
            // Collect 2 samples - each with 1 second period
            // See: https://de.wikipedia.org/wiki/Load#Der_Load_Average_auf_Unix-Systemen
            $statData1 = _getServerLoadLinuxData();
            sleep(1);
            $statData2 = _getServerLoadLinuxData();

            if
            (
                (!is_null($statData1)) &&
                (!is_null($statData2))
            ) {
                // Get difference
                $statData2[0] -= $statData1[0];
                $statData2[1] -= $statData1[1];
                $statData2[2] -= $statData1[2];
                $statData2[3] -= $statData1[3];

                // Sum up the 4 values for User, Nice, System and Idle and calculate
                // the percentage of idle time (which is part of the 4 values!)
                $cpuTime = $statData2[0] + $statData2[1] + $statData2[2] + $statData2[3];

                // Invert percentage to get CPU time, not idle time
                $load = 100 - ($statData2[3] * 100 / $cpuTime);
            }
        }

    }

    return $load;
}


//--- ram usage e.g. (values in kb !)
/*
Array
(
    [MemTotal] => 8102684000
    [MemFree] => 2894508000
//optional
    [MemAvailable] => 4569396000
    [SwapTotal] => 4194300000
    [SwapFree] => 4194300000
)
 */

//from https://stackoverflow.com/questions/1455379/get-server-ram-with-php

function wmiWBemLocatorQuery($query)
{
    if (class_exists('\\COM')) {
        try {
            $WbemLocator = new \COM("WbemScripting.SWbemLocator");
            $WbemServices = $WbemLocator->ConnectServer('127.0.0.1', 'root\CIMV2');
            $WbemServices->Security_->ImpersonationLevel = 3;
            // use wbemtest tool to query all classes for namespace root\cimv2
            return $WbemServices->ExecQuery($query);
        } catch (\com_exception $e) {
            echo $e->getMessage();
        }
    } elseif (!extension_loaded('com_dotnet'))
        trigger_error('It seems that the COM is not enabled in your php.ini', E_USER_WARNING);
    else {
        $err = error_get_last();
        trigger_error($err['message'], E_USER_WARNING);
    }

    return false;
}

// _dir_in_allowed_path this is your function to detect if a file is withing the allowed path (see the open_basedir PHP directive)
function _dir_in_allowed_path()
{
    //TODO
    return TRUE;
}


function getSystemMemoryInfo($output_key = '')
{
    $keys = array('MemTotal', 'MemFree', 'MemAvailable', 'SwapTotal', 'SwapFree');
    $result = array();

    try {
        // LINUX

        if (strtoupper(substr(PHP_OS, 0, 3)) === 'DAR') {

            $totalMemInBytes = shell_exec("sysctl -n hw.memsize");
            $totalMemInKb = $totalMemInBytes / 1024;

            $result['MemTotal'] = $totalMemInKb;

            //see http://badrit.com/blog/2014/5/1/how-to-calc-memory-usage-in-mavericks-programmatically#.WkUkIyOX-i4

            $freeMem = shell_exec('vm_stat | sed -n 1,2p');

            $freeMem2 = explode("\n", $freeMem);

            $part1 = str_replace("Mach Virtual Memory Statistics: (page size of ", '', $freeMem2[0]);
            $part2 = str_replace(' bytes)', '', $part1);
            $pageSizeInBytes = intval($part2);

            $freeMemString = str_replace('Pages free:', '', str_replace('.', '', $freeMem2[1]));
            $freeMemInPages = intval($freeMemString);


            $result['MemFree'] = $freeMemInPages * $pageSizeInBytes / 1024;

        } else if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
            {
                //TODO this gives kb???
                $wmi_found = false;
                if ($wmi_query = wmiWBemLocatorQuery(
                    "SELECT FreePhysicalMemory,FreeVirtualMemory,TotalSwapSpaceSize,TotalVirtualMemorySize,TotalVisibleMemorySize FROM Win32_OperatingSystem")) {
                    foreach ($wmi_query as $r) {
                        $result['MemFree'] = $r->FreePhysicalMemory * 1024;
                        $result['MemAvailable'] = $r->FreeVirtualMemory * 1024;
                        $result['SwapFree'] = $r->TotalSwapSpaceSize * 1024;
                        $result['SwapTotal'] = $r->TotalVirtualMemorySize * 1024;
                        $result['MemTotal'] = $r->TotalVisibleMemorySize * 1024;
                        $wmi_found = true;
                    }
                }
                // TODO a backup implementation using the $_SERVER array
            }
        else {//(strtoupper(substr(PHP_OS, 0, 3)) === 'LIN') { //LINUX
                $proc_dir = '/proc/';
                $data = _dir_in_allowed_path($proc_dir) ? @file($proc_dir . 'meminfo') : false;
                if (is_array($data))
                    foreach ($data as $d) {
                        if (0 == strlen(trim($d)))
                            continue;
                        $d = preg_split('/:/', $d);
                        $key = trim($d[0]);
                        if (!in_array($key, $keys))
                            continue;
                        $value = floatval(trim(str_replace(' kB', '', $d[1])));
                        $result[$key] = $value;
                    }
            }
        }
    catch
        (Exception $e) {
            echo $e->getMessage();
        }
    return empty($output_key) || !isset($result[$output_key]) ? $result : $result[$output_key];
}

$cpuUsageInPercentage = getServerLoad();


$tempMemory = getSystemMemoryInfo();

if ($tempMemory === NULL) {
    //$tempMemory = -1;
}

$totalRam = -1;
$freeRam = -1;

if (isset($tempMemory)) {

    if (isset($tempMemory['MemTotal']))
        $totalRam = $tempMemory['MemTotal'];

    if (isset($tempMemory['MemFree']))
        $freeRam = $tempMemory['MemFree'];
}

$message =
    $cpuUsageInPercentage . ';'
    . $totalRam . ';'
    . $freeRam;

$fe = fopen('php://stdout', 'w');
fwrite($fe, $message);
fclose($fe);

exit(0);
